use crate::{
	physics::{Pfloat, Polar},
	Doodad, Sprite, Spriteable, STARFIELD_COUNT, STARFIELD_PARALLAX_MAX, STARFIELD_PARALLAX_MIN,
	STARFIELD_SIZE,
};
use cgmath::{Deg, Point2, Vector2};
use rand::{self, Rng};

pub struct StarfieldStar {
	/// Star coordinates, in world space.
	/// These are relative to the center of a starfield tile.
	pub pos: Point2<Pfloat>,

	pub parallax: Pfloat,
	pub height: Pfloat,

	/// Color/brightness variation.
	/// See shader.
	pub tint: Vector2<Pfloat>,
}

pub struct System {
	bodies: Vec<Doodad>,
	pub starfield: Vec<StarfieldStar>,
}

impl System {
	pub fn new() -> Self {
		let mut rng = rand::thread_rng();
		let sz = STARFIELD_SIZE as f32 / 2.0;
		let mut s = System {
			bodies: Vec::new(),
			starfield: (0..STARFIELD_COUNT)
				.map(|_| StarfieldStar {
					pos: Point2 {
						x: rng.gen_range(-sz..=sz),
						y: rng.gen_range(-sz..=sz),
					},
					parallax: rng.gen_range(STARFIELD_PARALLAX_MIN..STARFIELD_PARALLAX_MAX),
					height: rng.gen_range(0.2..0.8), // TODO: configurable
					tint: Vector2 {
						x: rng.gen_range(0.0..=1.0),
						y: rng.gen_range(0.0..=1.0),
					},
				})
				.collect(),
		};

		s.bodies.push(Doodad {
			pos: (0.0, 0.0).into(),
			sprite: "a0".to_owned(),
			parallax: 10.0,
			height: 80.0,
		});

		s.bodies.push(Doodad {
			pos: Polar {
				center: (0.0, 0.0).into(),
				radius: 5000.0,
				angle: Deg { 0: 31.0 },
			}
			.to_cartesian(),
			sprite: "earth".to_owned(),
			parallax: 5.0,
			height: 120.0,
		});

		s.bodies.push(Doodad {
			pos: Polar {
				center: (0.0, 0.0).into(),
				radius: 200.0,
				angle: Deg { 0: 270.0 },
			}
			.to_cartesian(),
			sprite: "small".to_owned(),
			parallax: 0.5,
			height: 50.0,
		});

		return s;
	}

	pub fn sprites(&self) -> Vec<Sprite> {
		return self.bodies.iter().map(|x| x.sprite()).collect();
	}
}
